/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre: common3D.h      Version: 0.5                  Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: 
     Funciones para la matematica de puntos en 3D, asi como la
     definicion del estandar de putno flotante utilizado por la libreria
     tambien se ecuentran las funciones para el manejo de color por RGB.   
  Tipos/Clases: 
     FPOINT    objeto de punto flotante.         
     Point3D   Punto en el espacio R3.
     SRGB      Estructura para el manejo del espacio de color RGB.
  Funciones:
     Las funciones definidas en este modulo estan definidas por cada tipo 
     de objeto (ver mas adelantre en la definicion de cada uno).   
  Observaciones: 
     FPOINT corresponde al tipo estandar double, pero se puede modificar
     facilmente a otro formato (p.ej. punto fijo para eficiencia, todo
     programa que utilize la libreria al declarar una variable de punto
     flotante deberia utilizar este objeto para consistencia.
     
     SRGB corraponde al manjo del espacio RGB, pero por el momento este
     equivale a un punto 3D (3 valores de tipo FPOINT), por lo tanto esta
     clase se manipula por medio de las de la clase Point3D. 

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/

#ifndef __COMMON3D_H__
#define __COMMON3D_H__
#include <stdlib.h>
#include <math.h>
#include <values.h>

/*------------------  Boolean Constants --------------------------------*/
#ifndef TRUE
#define TRUE            1
#endif  TRUE
#ifndef FALSE
#define FALSE           0
#endif  FALSE
/*----------------------------------------------------------------------*/

/*------------------  Object Floating Point ----------------------------*/
/*----------------------------------------------------------------------- 
   Nombre: FPOINT           Version 0.5                Fecha: Abril-1996
   Descripcion: 
     Este objeto (clase) define el tipo de punto flotante a utilizar por
     la libreria, asi como un conjunto de funciones sobre el mismo.   
   Observaciones:
     FPOINT corresponde al tipo estandar double, pero se puede modificar
     facilmente a otro formato (p.ej. punto fijo para eficiencia, todo
     programa que utilize la libreria al declarar una variable de punto
     flotante deberia utilizar este objeto para consistencia.
   Autor: Daniel Barrero                                      Abril-1996 
 -----------------------------------------------------------------------*/ 
#define FPOINT               double
#define MAX_FPOINT           MAXDOUBLE 
#define MIN_FPOINT           MINDOUBLE 
#define FERROR               (FPOINT)1e-06 
#define FSQR(x)              (x*x)
#define FSQRT(x)             ((FPOINT)sqrt((double)x))
#define FABS(x)              fabs(x) 
#define FEQ(x,y)             (FABS(x-y)<=FERROR)
#define STRTOFPOINT(x)       (strtod(x,NULL))
/*----------------------------------------------------------------------*/

/*------------------  Object Point3D -----------------------------------*/
/*----------------------------------------------------------------------- 
   Nombre: Point3D          Version 0.5                Fecha: Abril-1996
   Descripcion: 
     AHHHHHH.....................
   Observaciones: 
   Autor: Daniel Barrero                                      Abril-1996 
 -----------------------------------------------------------------------*/ 
typedef struct  Point3D_s {
  FPOINT x;
  FPOINT y;
  FPOINT z;
} Point3D;

Point3D * newPoint3D ( FPOINT x, FPOINT y, FPOINT z);
Point3D * deletePoint3D ( Point3D * p);
Point3D * setPoint3D(Point3D * p, FPOINT x, FPOINT y, FPOINT z); 
Point3D * copyPoint3D(Point3D * p1, Point3D * p2);
Point3D * addPoint3D ( Point3D * p1, Point3D *p2);
Point3D * subPoint3D ( Point3D * p1, Point3D *p2);
#define multPoint3D(p1,p2)   ((p1!=NULL && p2!=NULL)?((p1->x*p2->x)+(p1->y*p2->y)+(p1->z*p2->z)):0;
Point3D * addPoint3DFPOINT ( Point3D * p, FPOINT s);
Point3D * multPoint3DFPOINT ( Point3D * p, FPOINT f);
#define distPoint3D(p1,p2)   ((p1!=NULL && p2!=NULL)?(FSQRT(FSQR(p2->x-p1->x) + FSQR(p2->y-p1->y) + FSQR(p2->z-p1->z)))?-1)
#define eqPoint3D(p1,p2)     ((p1==p2)?TRUE:((p1!=NULL && p2!=NULL) && (FEQ(p1->x,p2->x) && FEQ(p1->y,p2->y) && FEQ(p1->z,p2->z)))) 
/*----------------------------------------------------------------------*/

/*------------------  Object SRGB --------------------------------------*/
/*----------------------------------------------------------------------- 
   Nombre: SRGB             Version 0.5                Fecha: Abril-1996
   Descripcion: 
   Observaciones:
     SRGB corraponde al manjo del espacio RGB, pero por el momento este
     equivale a un punto 3D (3 valores de tipo FPOINT), por lo tanto esta
     clase se manipula por medio de las de la clase Point3D. 
   Autor: Daniel Barrero                                      Abril-1996 
 -----------------------------------------------------------------------*/ 
#define SRGB   Point3D 
/*----------------------------------------------------------------------*/

/*------------------  Object Surface  ----------------------------------*/
/*----------------------------------------------------------------------- 
   Nombre: Surface          Version 0.6                Fecha: Abril-1996
   Descripcion: 
   Observaciones:
   Autor: Daniel Barrero                                      Abril-1996 
 -----------------------------------------------------------------------*/  
typedef struct Surface_s {
  FPOINT ambient;            /* ambient */
  FPOINT diffuse;            /* diffuse */
  FPOINT brilliance;         /* brilliance */
  FPOINT specular;           /* specular RGB */
  FPOINT roughness;          /* specular coef */
  FPOINT reflection;         /* reflection 0.0 - 1.0 */
  FPOINT refraction;         /* refraction 0.0 - 1.0 */
} Surface;

Surface * newSurface(FPOINT ambient,FPOINT diffuse, FPOINT brilliance,
                     FPOINT specular,FPOINT roughness,FPOINT reflection,
                     FPOINT refraction);
Surface * deleteSurface( Surface * surf);
Surface * setSurface(Surface * surf, FPOINT ambient,FPOINT diffuse, 
                     FPOINT brilliance,FPOINT specular,FPOINT roughness,
                     FPOINT reflection,FPOINT refraction);
Surface * copySurface(Surface * surf1, Surface * surf2);
#endif __COMMON3D_H__

