/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/
/*************************************************************************
  Nombre:      libdxf.h  Version: 0.5                   Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: Rutinas para leer un archivo DXF y generacion de una lista 
               de poligonos con la informacion.
  Tipos:       DXF_file ( filename, file_handle, ....).
  Funciones:   
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#ifndef __LIB_DXF__
#define __LIB_DXF__
#include <stdio.h>
#include "common3d.h" 
#include "libpolymesh.h"
#include "extrafuncs.h"
#include "dxftokens.h"


/*------------------ Object DXF_file -----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: DXF_file         Version 0.5                Fecha: Abril-1996
   Descripcion:
     AHHHHHH.....................
   Observaciones:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/
#define DXF_TEXT    0
#define DXF_BINARY  1
#define DXF_IGES    2
#define DXF_READ    0
#define DXF_WRITE   1
#define DXF_OPENED  2
#define DXF_CLOSED  3
typedef struct DXF_file_s {
  char * filename;
  FILE * file_handle;
  char mode;                   /* Mode: TEXT,BINARY */ 
  char state;                  /* State: READ,WRITE,OPENED,CLOSED*/
  SceneObj * objects;          /* Scene containing the list of objects in 
                                  polymeshed form */ 
} DXF_file;

DXF_file * newDXF(char * filename, char mode);
DXF_file * deleteDXF(DXF_file * dxf);
DXF_file * openDXF(DXF_file * dxf);
DXF_file * closeDXF(DXF_file * dxf);
DXF_file * readDXF(DXF_file * dxf);
DXF_file * writeDXF(DXF_file * dxf);

/*----------------------------------------------------------------------*/

/*------------------ Object DXF_rule -----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: DXF_rule         Version 0.5                Fecha: Abril-1996
   Descripcion:
     Realiza el parsing de un archivo DXF (TEXTO).
   Observaciones:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/
#define funcRule(f)         char (*f)(DXF_file *,DXF_rule *, DXF_token *)

typedef struct DXF_rule_s DXF_rule;

struct DXF_rule_s {
   DXF_token * tok;
   int num_rules;
   DXF_rule ** lrules;
   int act_rule;
   funcRule(parse);
};

DXF_rule * newRuleDXF(DXF_token * tok,funcRule(parse));
DXF_rule * deleteRuleDXF( DXF_rule * rule);
int addRuleDXF(DXF_rule * rule, DXF_rule * rul);
int delRuleDXF(DXF_rule * rule);
DXF_rule * initParserRulesDXF();
DXF_file * parseDXF(DXF_file * dxf, DXF_rule * rule);

/*----------------------------------------------------------------------*/

#endif __LIB_DXF__
