/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:  LIB_POLYMESH    Version: 0.5                  Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: 
  Tipos/Clases:       
  Funciones:   
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/

#ifndef __LIB_POLYMESH__
#define __LIB_POLYMESH__
#include "common3d.h"

/*------------------  Object Polygon -----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: Polygon          Version 0.5                Fecha: Abril-1996
   Descripcion:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/

#define POL_CLOSED        0
#define POL_OPENED        1
#define POL_POINT_VALS    0 
#define POL_POINT_REFS    1

typedef struct Polygon_s {
  char pol_type;               /* Polygon type: CLOSED (default), 
                                                OPENED (polyline) */
  union {
     Point3D * pnts;    
     Point3D ** refs;
  } points;     /* List of points */
  char points_type;            /* List of POINT_VALS or POINT_REFS */
  int num_points;              /* Number of points */
  SRGB *color;                  /* Polygon color */
  Surface * surface;            /* Surface */
} Polygon;

/* Creator / Destructor Functions Object Polygon */
Polygon * newPolygon(char pol_type,char points_type,SRGB * color);
Polygon * deletePolygon ( Polygon * pol);

int addPolygonPoint(Polygon *pol,Point3D *p); 
int delPolygonPoint(Polygon *pol);
int insertPolygonPoint(Polygon *pol,Point3D *p,int pos);
int removePolygonPoint(Polygon *pol,int pos);
char changePolygonType(Polygon *pol,char pol_type);
char changePoligonPointsType(Polygon *pol,char points_type);

Point3D * maxPolygon(Polygon * pol);
Point3D * minPolygon(Polygon * pol);
/*----------------------------------------------------------------------*/

/*------------------  Object PolyMesh ----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: PolyMeshObj      Version 0.5                Fecha: Abril-1996
   Descripcion:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/
typedef struct PolyMeshObj_s {
     char * name;            /* Name of the polymesh object */
     Point3D ** points;        /* Points of the polymesh */
     int num_points;          /* Number of points in the  polymesh */
     Polygon ** polygons;      /* Polygons contructing the mesh */
     int num_polygons;        /* Number of polygons constructiong the mesh */
     SRGB *color;              /* General color of the polymesh */
     Surface * surface;            /* Surface */
} PolyMeshObj;
 
/* Creator / Destructor Funtions Object Polymesh) */
PolyMeshObj * newPolyMesh(char *name, SRGB * color);
PolyMeshObj * deletePolyMesh(PolyMeshObj * pmesh);
PolyMeshObj * changePolyMeshObjName(PolyMeshObj * pmesh, char * name);
int addPolyMeshObjPolygon(PolyMeshObj * pmesh, Polygon * pol);
int delPolyMeshObjPolygon(PolyMeshObj * pmesh);
int insertPolyMeshObjPolygon(PolyMeshObj * pmesh, Polygon * pol,int pos);
int removePolyMeshObjPolygon(PolyMeshObj * pmesh, int pos);
int addPolyMeshObjPoint(PolyMeshObj * pmesh, Point3D * p);
int delPolyMeshObjPoint(PolyMeshObj * pmesh);
int insertPolyMeshObjPoint(PolyMeshObj * pmesh, Point3D * p,int pos);
int removePolyMeshObjPoint(PolyMeshObj * pmesh, int pos);
int addPolyMeshObjPointEx(PolyMeshObj * pmesh, Point3D * p);

int addPolyMeshObjPolygonPoint(PolyMeshObj * pmesh, Point3D * p);
int delPolyMeshObjPolygonPoint(PolyMeshObj * pmesh);
int insertPolyMeshObjPolygonPoint(PolyMeshObj * pmesh, int pol_num, Point3D * p, int point_num); 
int removePolyMeshObjPolygonPoint(PolyMeshObj * pmesh, int pol_num, int point_num); 

Point3D * maxPolyMeshObj(PolyMeshObj * pmesh);
Point3D * minPolyMeshObj(PolyMeshObj * pmesh);

/*----------------------------------------------------------------------*/

/*------------------  Object SceneObj ----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: SceneObj         Version 0.5                Fecha: Abril-1996
   Descripcion:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/
typedef struct SceneObj_s {
     char * name;            /* Name of the scene */
     PolyMeshObj **objects;  /* Objects of the scene */
     int num_objects;        /* Number of objects in the scene */
     SRGB *color;            /* General ambient color of the scene  */
} SceneObj;
 
/* Creator / Destructor Funtions Object Polymesh) */
SceneObj * newScene(char *name, SRGB * color);
SceneObj * deleteScene(SceneObj *scene );
SceneObj * changeSceneObjName(SceneObj * scene, char * name);
int addSceneObjPolyMesh(SceneObj * scene, PolyMeshObj * pmesh);
int delSceneObjPolyMesh(SceneObj * scene);
int insertSceneObjPolyMesh(SceneObj * scene, PolyMeshObj * pmesh, int pos);
int removeSceneObjPolyMesh(SceneObj * scene, int pos);

Point3D * maxSceneObj(SceneObj * scene);
Point3D * minSceneObj(SceneObj * scene);
/*----------------------------------------------------------------------*/

#endif __LIB_POLYMESH__

