/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/
/*************************************************************************
  Nombre:   dxfinfo       Version: 0.5                   Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: Programa para preba de libdxf y polimeshes, lee un archivo
               DXF y da la informacion pertinente al mismo:
                   Numero de Poligonos,
                   Numero de Objetos,
                   Numero de Puntos,
                   Viewpoint,
                   MinyMax de la Escena, ...
  Tipos:
  Funciones:
  Observaciones:  Requiere libdxf 
 
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996
**************************************************************************/

/* --------------------- In pre-pre-alpha state :) -------*/
#include <stdio.h>
#include "libdxf.h"

char * DXFFileName = NULL;   
 
void printUsage()
{
 printf("dxfinfo    - DXF file info by Daniel Barrero\n");
 printf("              e-mail:d-barrero@argos.uniandes.edu.co\n");
 printf("MOX - Centro de Computacion Avanzada en Ingenieria.\n");
 printf("\nUsage:\n");
 printf("  dxfinfo  filename \n"); 
}

char Args(int argc, char **argv)
{
 if(argc != 2) 
    return FALSE
 ;
 DXFFileName = argv[1];   
 return TRUE;
}

void printDXFInfo()
{
 DXF_file * dxf;
 Point3D *p;
 int i,numpol,numpoints;
 dxf=newDXF(DXFFileName,DXF_TEXT);
 if(readDXF(dxf) !=NULL && dxf!=NULL) { 
   /* imprimir informacion sobre el DXF */
   printf("DXF file info:\n");
   printf("  Filename:  %s \n",dxf->filename);
   printf("  FileType:  Autocad DXF %s file \n",(dxf->mode==DXF_IGES)?"DXF_IGES":(dxf->mode==DXF_TEXT)?"DXF_TEXT":"DXF_BINARY");
   if(dxf->objects!=NULL) {
      printf("  Scene Name:  %s\n",dxf->objects->name);
      printf("  Num Objects: %d\n",dxf->objects->num_objects);
      for(i=numpol=numpoints=0;i<dxf->objects->num_objects;i++) {
            numpol+= dxf->objects->objects[i]->num_polygons;
            numpoints+= dxf->objects->objects[i]->num_points;
      }
      printf("  Num Points  : %d\n",numpoints);
      printf("  Num Polygons: %d\n",numpol);
      p=minSceneObj(dxf->objects);
      printf("  Scene Min:   %f, %f, %f\n",p->x,p->y,p->z);
      deletePoint3D(p);
      p=maxSceneObj(dxf->objects);
      printf("  Scene Max:   %f, %f, %f\n",p->x,p->y,p->z);
      deletePoint3D(p);
   }
 } else {
   deleteDXF(dxf);
   printf("Error: Bad or missing DXF file \n");
   exit(0);
 } 
 deleteDXF(dxf);
}

void main(int argc, char **argv)
{
 if (Args(argc, argv) == FALSE) {
    printUsage();
    exit(1);
 }
 printDXFInfo();
}

