/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:      LIBDXF    Version: 0.5                   Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: Rutinas para leer un archivo DXF y generacion de una lista 
               de poligonos con la informacion.
  Tipos:       DXF_file ( filename, file_handle, ....).
  Funciones:  
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "extrafuncs.h"
#include "libdxf.h"
#include "parsefuncs.h"

/*------------------ Object DXF_file -----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: DXF_file         Version 0.5                Fecha: Abril-1996
   Descripcion:
     AHHHHHH.....................
   Observaciones:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/

DXF_file * newDXF(char * filename, char mode)
{
 DXF_file * dxf;
 dxf=(DXF_file *)malloc(sizeof(DXF_file));
 dxf->filename=strdup(filename);
 dxf->mode=mode;
 dxf->state=DXF_CLOSED;
 dxf->objects=NULL;
 dxf->file_handle=NULL;

 return dxf;
}

DXF_file * deleteDXF(DXF_file * dxf)
{
 if(dxf==NULL)
   return NULL
 ;
 free(dxf->filename);
 if(dxf->file_handle!=NULL) 
   fclose(dxf->file_handle)
 ;
 dxf->file_handle=NULL;
 deleteScene(dxf->objects);
 free(dxf);
 dxf=NULL;

 return dxf;
} 

DXF_file * openDXF(DXF_file * dxf)
{
 if(dxf==NULL) 
    return NULL
 ;
 if(dxf->state!=DXF_CLOSED)
   return NULL
 ;
 if(dxf->mode==DXF_TEXT && (dxf->file_handle=fopen(dxf->filename,"r+"))==NULL)
   return NULL
 ;
 if(dxf->mode==DXF_BINARY && (dxf->file_handle=fopen(dxf->filename,"rb+"))==NULL)
   return NULL 
 ;
 dxf->state=DXF_OPENED; 
 return dxf;
}

DXF_file * closeDXF(DXF_file * dxf)
{
 if(dxf==NULL) 
   return NULL
 ;
 if(dxf->state!=DXF_CLOSED)
    fclose(dxf->file_handle)
 ;
 dxf->state=DXF_CLOSED;
 return dxf;
}

DXF_file * readDXF(DXF_file * dxf)
{
 DXF_rule * rule;
 if(dxf==NULL)
   return NULL
 ;
 if(dxf->state==DXF_CLOSED && openDXF(dxf)==NULL)  
   return NULL
 ;
 dxf->state=DXF_READ;
 if(dxf->mode==DXF_TEXT) {
    rule= initParserRulesDXF();
    if(rule==NULL || parseDXF(dxf,rule)==NULL) {
       if(rule!=NULL)
          deleteRuleDXF(rule)
       ;
       return NULL;
    }
 } 
/* --------- DXF bin (DXB) -------
 if(dxf->mode==DXF_BINARY) {
    rule= initParserRulesDXB();
    if(rule==NULL || parseDXB(dxf,rule)==NULL) {
       if(rule!=NULL)
          deleteRuleDXB(rule)
       ;
       return NULL;
    }
 }
----------------------------------*/
 return dxf;
}

DXF_file * writeDXF(DXF_file * dxf)
{
 if(dxf==NULL)
   return NULL
 ;
 printf("Advertencia: Opcion en proceso de desarrollo (funcion no publicada!)\n");
 return dxf;
}

/*----------------------------------------------------------------------*/

/*------------------ Object DXF_rule -----------------------------------*/
/*-----------------------------------------------------------------------
   Nombre: DXF_rule         Version 0.5                Fecha: Abril-1996
   Descripcion:
     Realiza el parsing de un archivo DXF (TEXTO).
   Observaciones:
   Autor: Daniel Barrero                                      Abril-1996
 -----------------------------------------------------------------------*/

DXF_rule * newRuleDXF(DXF_token * tok,funcRule(parse))
{
 DXF_rule * rule;
 rule=( DXF_rule *)malloc(sizeof(DXF_rule));
 rule->tok=(tok!=NULL)?newTokenDXF(tok->gr_id,tok->data,tok->datalen):
                       newTokenDXF(COMMENT,"COMMENT",strlen("COMMENT"));
 rule->parse=parse;
 rule->act_rule=-1;
 rule->num_rules=0;
 rule->lrules=NULL;
 return rule;
}

DXF_rule * deleteRuleDXF( DXF_rule * rule)
{
 int i;
 if(rule==NULL)
   return NULL
 ;
 for(i=0;i<rule->num_rules; i ++) 
    deleteRuleDXF(rule->lrules[i])
 ;
 deleteTokenDXF(rule->tok);
 free(rule);
 rule=NULL;
 return rule;
}

int addRuleDXF(DXF_rule * rule, DXF_rule * rul)
{
 DXF_rule **rules;
 if(rule==NULL || rul==NULL)  
    return -1 
 ;
 rule->num_rules++;
 rules=(DXF_rule **) realloc (rule->lrules,sizeof(DXF_rule*)*(rule->num_rules));
 if(rules!=NULL) {
   rules[rule->num_rules-1]=rul;
   rule->lrules=rules;
 } else 
   rule->num_rules--
 ;
 return rule->num_rules;
}

int delRuleDXF(DXF_rule * rule)
{
 DXF_rule   *rul; 
 if(rule==NULL)  
    return -1 
 ;
 if(rule->num_rules==0)
   return 0
 ;
 rule->num_rules--;  
 rul=rule->lrules[rule->num_rules];
 deleteRuleDXF(rul);
 rule->lrules=(DXF_rule  **) realloc(rule->lrules,sizeof(DXF_rule*)*(rule->num_rules));

 return rule->num_rules;    
}

DXF_rule * initParserRulesDXF()
{
 DXF_rule * rule;
 DXF_token * tok;
 DXF_rule * nrule,*nrule1,*nruletmp;
 tok=newTokenDXF(DXF_FILE,"DXF_FILE",strlen("DXF_FILE"));  
/* funcion de la regla primaria de DXF archivo dxf */
 if((rule=newRuleDXF(tok,parseDXFfile))==NULL)
   return NULL
 ;
/*--------- header   funcs -----------*/
 deleteTokenDXF(tok);
 tok=newTokenDXF(NAME0,"HEADER",strlen("HEADER"));  
 nrule=newRuleDXF(tok,parseDXFHEADER);
 addRuleDXF(rule,nrule);

/*--------- entities funcs -----------*/
 deleteTokenDXF(tok);
 tok=newTokenDXF(NAME0,"ENTITIES",strlen("ENTITIES"));  
 nrule=newRuleDXF(tok,parseDXFENTITIES);
 deleteTokenDXF(tok);
 tok=newTokenDXF(START,"3DFACE",strlen("3DFACE"));  
 nrule1=newRuleDXF(tok,parseDXF3DFACE);
 deleteTokenDXF(tok);
 tok=newTokenDXF(X0,"X0",strlen("X0"));
 nruletmp=newRuleDXF(tok,parseDXFXVALS);
 addRuleDXF(nrule1,nruletmp);

 deleteTokenDXF(tok);
 tok=newTokenDXF(Y0,"Y0",strlen("Y0"));
 nruletmp=newRuleDXF(tok,parseDXFYVALS);
 addRuleDXF(nrule1,nruletmp);

 deleteTokenDXF(tok);
 tok=newTokenDXF(Z0,"Z0",strlen("Z0"));
 nruletmp=newRuleDXF(tok,parseDXFZVALS);
 addRuleDXF(nrule1,nruletmp);

 addRuleDXF(nrule,nrule1);
 addRuleDXF(rule,nrule);

/* --------- blocks funcs -----------*/

 deleteTokenDXF(tok);
 tok=newTokenDXF(NAME0,"BLOCKS",strlen("BLOCKS"));
 nrule=newRuleDXF(tok,parseDXFBLOCKS);

 deleteTokenDXF(tok);
 tok=newTokenDXF(START,"3DFACE",strlen("3DFACE"));  
 nrule1=newRuleDXF(tok,parseDXF3DFACE);

 deleteTokenDXF(tok);
 tok=newTokenDXF(X0,"X0",strlen("X0"));
 nruletmp=newRuleDXF(tok,parseDXFXVALS);
 addRuleDXF(nrule1,nruletmp);

 deleteTokenDXF(tok);
 tok=newTokenDXF(Y0,"Y0",strlen("Y0"));
 nruletmp=newRuleDXF(tok,parseDXFYVALS);
 addRuleDXF(nrule1,nruletmp);

 deleteTokenDXF(tok);
 tok=newTokenDXF(Z0,"Z0",strlen("Z0"));
 nruletmp=newRuleDXF(tok,parseDXFZVALS);
 addRuleDXF(nrule1,nruletmp);

 addRuleDXF(nrule,nrule1);
 addRuleDXF(rule,nrule);
 
 return rule;
}

DXF_file * parseDXF(DXF_file * dxf, DXF_rule * rule)
{
 DXF_token * tok; 
 char resp;
 if(dxf==NULL || rule==NULL)
   return NULL
 ;
 if((dxf->state!=DXF_OPENED && dxf->state!=DXF_READ) || dxf->mode!=DXF_TEXT) 
  return NULL
 ;
 for(resp=TRUE;((tok=readTokenDXF(dxf->file_handle))!=NULL) && resp;) { 
     resp=(*(rule->parse))(dxf,rule,tok);
 }   
 if(!resp || dxf->objects==NULL || 
    (dxf->objects!=NULL && dxf->objects->num_objects==0)) 
   return NULL
 ;

 return dxf;
}

/*----------------------------------------------------------------------*/

