/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:  parsefuncs.c  Version: 0.6                   Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: Rutinas para procesar cada uno de los tokens de un archivo 
               DXF                             
  Tipos:       
  Funciones:  
  Observaciones: las funciones deben ser de la forma:
                 char func_name(DXF_file *, DXF_rule *, DXF_token *)  
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "extrafuncs.h"
#include "libdxf.h"
#include "parsefuncs.h"

DXF_token * toksect=NULL,*tokendsect=NULL,*tokeof=NULL;
char parseDXFfile(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 char resp;
/*printf("tok bas bas%s\n",tok->data);*/
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(toksect==NULL)
   toksect=newTokenDXF(START,"SECTION",strlen("SECTION"))
 ;
 if(tokendsect==NULL)
   tokendsect=newTokenDXF(START,"ENDSEC",strlen("ENDSEC"))
 ;
 if(tokeof==NULL)
   tokeof=newTokenDXF(START,"EOF",strlen("EOF"))
 ;
 if( eqTokenDXF(tok,tokeof)!=TOK_EQ ){
   if( eqTokenDXF(tok,toksect)==TOK_EQ ||  eqTokenDXF(tok,tokendsect)==TOK_EQ ){
      rule->act_rule=-1;
   } else if(rule->act_rule==-1)  {
     for(resp=FALSE, rule->act_rule=0; rule->act_rule<rule->num_rules && !resp ; rule->act_rule++){ 
        rule->lrules[rule->act_rule]->act_rule=-1;
/*printf("section : %s \n",rule->lrules[rule->act_rule]->tok->data);*/
        resp=(*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok);
     }
     if(rule->act_rule==rule->num_rules) {
        rule->act_rule=-1;
     } 
   } else {
        (*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok);
   }
 } else {
   deleteTokenDXF(toksect);
   deleteTokenDXF(tokendsect);
   deleteTokenDXF(tokeof);
 }
 return TRUE;
} 

char parseDXFHEADER(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(eqTokenDXF(rule->tok,tok)==TOK_EQ ){
printf("----------- processing header ------\n");
   rule->act_rule=-2;
   return TRUE;
 } else if(rule->act_rule==-2) {
   return TRUE;
 }
 return FALSE;
}

char parseDXFENTITIES(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 char resp;
/*printf("tok bas%s\n",tok->data);*/
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(eqTokenDXF(rule->tok,tok)==TOK_EQ ){
printf("----------- new entities polymesh ------\n");
   rule->act_rule=-1;
   if(dxf->objects==NULL) { 
      dxf->objects=newScene(dxf->filename,newPoint3D(1,1,1));
   }
   addSceneObjPolyMesh(dxf->objects,newPolyMesh(tok->data,newPoint3D(1,1,1)));
 } else if ( rule->act_rule==-1 ) {
     for(rule->act_rule=0; rule->act_rule<rule->num_rules; rule->act_rule++) {
        rule->lrules[rule->act_rule]->act_rule=-1;
     }
     for(rule->act_rule=0; rule->act_rule<rule->num_rules &&
        !(*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok) ;
        rule->act_rule++)
     ;
     if(rule->act_rule==rule->num_rules) {
        rule->act_rule=-1;
        return FALSE;
     }
 } else {
     resp=(*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok);
     if(!resp)
         rule->act_rule=-1
     ;
     return resp;
 }
 return TRUE;
}

char parseDXFBLOCKS(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 char resp;
/*printf("tok bas%s\n",tok->data);*/
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(eqTokenDXF(rule->tok,tok)==TOK_EQ ){
printf("----------- new blocks polymesh ------\n");
   rule->act_rule=-1;
   if(dxf->objects==NULL) { 
      dxf->objects=newScene(dxf->filename,newPoint3D(1,1,1));
   }
   addSceneObjPolyMesh(dxf->objects,newPolyMesh(tok->data,newPoint3D(1,1,1)));
 } else if ( rule->act_rule==-1 ) {
     for(rule->act_rule=0; rule->act_rule<rule->num_rules; rule->act_rule++) {
        rule->lrules[rule->act_rule]->act_rule=-1;
     }
     for(rule->act_rule=0; rule->act_rule<rule->num_rules &&
        !(*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok) ;
        rule->act_rule++)
     ;
     if(rule->act_rule==rule->num_rules) {
        rule->act_rule=-1;
        return FALSE;
     }
 } else {
     resp=(*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok);
     if(!resp)
         rule->act_rule=-1
     ;
     return resp;
 }
 return TRUE;
}

Point3D * __PointActTok=NULL;
int    __CountCoordTok;
char parseDXF3DFACE(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 char tmp;
 int posp;
/*
printf("tok face %s\n",tok->data);
printf("tok base face %s\n",tok->data);
*/
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if((tmp=eqTokenDXF(rule->tok,tok))==TOK_DATA_EQ || tmp==TOK_EQ ){
/*printf("en 3d face inicio entitie tok %s \n",tok->data);*/
     rule->act_rule=0;
   __CountCoordTok=0;
   __PointActTok=newPoint3D(0,0,0);
   if(dxf->objects==NULL) {
      dxf->objects=newScene(dxf->filename,newPoint3D(1,1,1));
      addSceneObjPolyMesh(dxf->objects,newPolyMesh(tok->data,newPoint3D(1,1,1)));
   }
   addPolyMeshObjPolygon(dxf->objects->objects[dxf->objects->num_objects-1],
                         newPolygon(POL_CLOSED, POL_POINT_REFS,newPoint3D(1,1,1)));
 } else if(rule->act_rule!=-1) {
/*printf("en 3d face making points data %s \n",tok->data);*/
     for(rule->act_rule=0; rule->act_rule<rule->num_rules &&
        !((*(rule->lrules[rule->act_rule]->parse))(dxf,rule->lrules[rule->act_rule],tok)); rule->act_rule++)
     ;
   if(__PointActTok!=NULL && rule->act_rule<rule->num_rules && __CountCoordTok>=3) {

      posp=addPolyMeshObjPointEx(dxf->objects->objects[dxf->objects->num_objects-1],__PointActTok);
       if(posp < dxf->objects->objects[dxf->objects->num_objects-1]->num_points) {
          deletePoint3D(__PointActTok);
          __PointActTok=dxf->objects->objects[dxf->objects->num_objects-1]->points[posp];
       }

/*
     addPolyMeshObjPoint(dxf->objects->objects[dxf->objects->num_objects-1],__PointActTok);
*/
      addPolyMeshObjPolygonPoint(
                dxf->objects->objects[dxf->objects->num_objects-1],
                __PointActTok);
/*printf("[ %f, %f ,%f ]\n",__PointActTok->x,__PointActTok->y,__PointActTok->z);*/
        __CountCoordTok=0;
        __PointActTok=NULL;
/*-------------------
printf(" num points polygon %d \n",
       dxf->objects->objects[dxf->objects->num_objects-1]->polygons[dxf->objects->objects[dxf->objects->num_objects-1]->num_polygons-1]->num_points);
printf(" num polygons %d \n",dxf->objects->objects[dxf->objects->num_objects-1]->num_polygons);
 --------------------*/
        if(dxf->objects->objects[dxf->objects->num_objects-1]->
                polygons[dxf->objects->objects[dxf->objects->num_objects-1]->
                num_polygons-1]->num_points>=4) {
          return FALSE;
          rule->act_rule=-1;
        } 
     }
 } else {
   return FALSE;
 }
 return TRUE;
}

char parseDXFXVALS(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(!(tok->gr_id>=X0 && tok->gr_id<=XN)) 
   return FALSE
 ;
 if(__PointActTok==NULL && __CountCoordTok==0) {
    __PointActTok=newPoint3D(0,0,0);
 }
 if(__CountCoordTok==0) {
    __CountCoordTok++; 
    __PointActTok->x=STRTOFPOINT(tok->data);
 } 
 return TRUE;
}

char parseDXFYVALS(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(!(tok->gr_id>=Y0 && tok->gr_id<=YN)) 
   return FALSE
 ;
 if(__PointActTok==NULL && __CountCoordTok==1) {
    __PointActTok=newPoint3D(0,0,0);
 }
 if(__CountCoordTok==1) {
    __CountCoordTok++; 
    __PointActTok->y=STRTOFPOINT(tok->data);
 } 
 return TRUE;
}

char parseDXFZVALS(DXF_file *dxf, DXF_rule *rule, DXF_token *tok)
{
 if(dxf==NULL || rule==NULL || tok ==NULL)
   return FALSE
 ;
 if(!(tok->gr_id>=Z0 && tok->gr_id<=ZN)) 
   return FALSE
 ;
 if(__PointActTok==NULL && __CountCoordTok==2) {
    __PointActTok=newPoint3D(0,0,0);
 }
 if(__CountCoordTok==2) {
    __CountCoordTok++; 
    __PointActTok->z=STRTOFPOINT(tok->data);
 } 
 return TRUE;
}
