/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:  polygon.c    Version: 0.5                  Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: 
  Tipos/Clases:       
  Funciones:   
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "extrafuncs.h"
#include "common3d.h"
#include "libpolymesh.h"

/*------------------  Object Polygon -----------------------------------*/
Polygon * newPolygon(char pol_type,char points_type,SRGB * color)
{
 Polygon *p = (Polygon *) malloc(sizeof(Polygon)); 
 p->pol_type=pol_type;
 p->points_type=points_type;
 p->points.pnts=NULL;
 p->num_points=0;
 p->color=color;
 p->surface=NULL; 

 return p;
}

Polygon * deletePolygon ( Polygon * pol)
{
 if(pol==NULL) 
    return NULL
 ;
 if(pol->points_type==POL_POINT_VALS) 
       free(pol->points.pnts)
 ; else {  
       for(;pol->num_points > 0; pol->num_points--)  
           pol->points.refs[pol->num_points-1]=NULL
       ;
       free(pol->points.refs); 
 }
 deletePoint3D(pol->color);
 deleteSurface(pol->surface);
 free(pol);
 pol=NULL;
 return pol;  
}

int addPolygonPoint(Polygon *pol,Point3D *p) 
{
 Point3D *pnts;
 Point3D **refs;
 if(pol==NULL || p==NULL )  
    return -1 
 ;
 if(pol->points_type==POL_POINT_VALS)  {
    pol->num_points++;
    pnts=(Point3D *) realloc (pol->points.pnts,sizeof(Point3D)*(pol->num_points));
    if(pnts!=NULL) {  
       copyPoint3D(&(pnts[pol->num_points-1]),p);
       pol->points.pnts=pnts;
    } else 
      pol->num_points--
    ;
 } else { 
    pol->num_points++;
    refs=(Point3D **) realloc (pol->points.refs,sizeof(Point3D*)*(pol->num_points));
    if(refs!=NULL) {
/*printf(" adding point to polygon\n");*/
       refs[pol->num_points-1]=p;
       pol->points.refs=refs;
    } else {
/*printf("not adding point to polygon\n");*/
      pol->num_points--;
    }
 }
 return pol->num_points;
}

int delPolygonPoint(Polygon *pol)
{
 if(pol==NULL)  
    return -1 
 ;
 if(pol->points_type==POL_POINT_VALS)  {
    pol->num_points--;
    pol->points.pnts=(Point3D *) realloc (pol->points.pnts,sizeof(Point3D)*(pol->num_points));
 } else { 
    pol->num_points--;
    pol->points.refs=(Point3D **) realloc (pol->points.refs,sizeof(Point3D*)*(pol->num_points));
 }
 return pol->num_points;
}

int insertPolygonPoint(Polygon *pol,Point3D *p,int pos)
{
 Point3D *pnts,tmp,n;
 Point3D **refs,*tmpr,*nr;
 if(pol==NULL || pos<0 || p==NULL)  
    return -1 
 ;
 if(pos>=pol->num_points) 
    return addPolygonPoint(pol,p)
 ;
 if(pol->points_type==POL_POINT_VALS)  {
    pol->num_points++;
    pnts=(Point3D *) realloc (pol->points.pnts,sizeof(Point3D)*(pol->num_points));
    if(pnts!=NULL) {
       for(copyPoint3D(&n,p);pos<pol->num_points; pos++) {
          copyPoint3D(&tmp,&(pnts[pos]));
          copyPoint3D(&(pnts[pos]),&n);
          copyPoint3D(&n,&tmp);
       } 
       pol->points.pnts=pnts;
    } else 
      pol->num_points--
    ;
 } else { 
    pol->num_points++;
    refs=(Point3D **) realloc (pol->points.refs,sizeof(Point3D*)*(pol->num_points));
    if(refs!=NULL) {
       for(nr=p;pos<pol->num_points; pos++) {
          tmpr=refs[pos];
          refs[pos]=nr;
          nr=tmpr;
       } 
       pol->points.refs=refs;
    } else 
      pol->num_points--
    ;
 }
 return pol->num_points;
}

int removePolygonPoint(Polygon *pol,int pos)
{
 Point3D *pnts,tmp,n;
 Point3D **refs,*tmpr,*nr;
 int i;
 if(pol==NULL || pos<0)  
    return -1 
 ;
 if(pos>=pol->num_points) 
    return delPolygonPoint(pol)
 ;
 if(pol->points_type==POL_POINT_VALS)  {
    pnts=pol->points.pnts;
    for(i=pol->num_points-2,copyPoint3D(&n,&(pnts[i+1]));i>=pos; i--) {
       copyPoint3D(&tmp,&(pnts[i]));
       copyPoint3D(&(pnts[i]),&n);
       copyPoint3D(&n,&tmp);
    } 
    pol->num_points--;
    pol->points.pnts=(Point3D *) realloc (pnts,sizeof(Point3D)*(pol->num_points));
 } else { 
    refs=pol->points.refs;
    for(i=pol->num_points-2,nr=refs[i+1];i>=pos;i--) {
       tmpr=refs[i];
       refs[i]=nr;
       nr=tmpr;
    } 
    pol->num_points--;
    pol->points.refs=(Point3D **) realloc (refs,sizeof(Point3D*)*(pol->num_points));
 }
 return pol->num_points;
}

char changePolygonType(Polygon *pol,char pol_type)
{
 if(pol==NULL)
   return FALSE
 ;
 pol->pol_type=pol_type;
 return TRUE;
}

char changePoligonPointsType(Polygon *pol,char points_type)
{
 Point3D *pnts=NULL; 
 Point3D **refs=NULL;
 int i;
 if(pol==NULL) 
    return FALSE
 ;
 if(points_type==pol->points_type || pol->num_points<=0) {
   pol->points_type=points_type; 
   return TRUE;
 } 
 if(points_type==POL_POINT_VALS && pol->points_type==POL_POINT_REFS) {
    pnts=(Point3D *) malloc( sizeof(Point3D)*(pol->num_points)); 
    for(i=0;i<pol->num_points; i++) 
       copyPoint3D(&(pnts[i]),pol->points.refs[i])
    ;
    free(pol->points.refs);
    pol->points.pnts=pnts;
    pol->points_type=points_type;
 } else {
    refs=(Point3D **) malloc( sizeof(Point3D*)*(pol->num_points));
    for(i=0;i<pol->num_points; i++)
       refs[i]=newPoint3D(pol->points.refs[i]->x,pol->points.refs[i]->y,pol->points.refs[i]->z)
    ;
    free(pol->points.pnts);
    pol->points.refs=refs;
    pol->points_type=points_type;
 }
 return TRUE;
}

Point3D * maxPolygon(Polygon * pol)
{
 Point3D * p ;
 int i;
 p = newPoint3D(0,0,0);
 if(pol==NULL || pol->num_points<=0) 
   return NULL
 ;
 if(pol->points_type==POL_POINT_VALS) {
    i=0;
    copyPoint3D(p,&(pol->points.pnts[i]));
    for(i=1; i<pol->num_points; i++) {
       p->x=(pol->points.pnts[i].x > p->x)?pol->points.pnts[i].x:p->x;
       p->y=(pol->points.pnts[i].y > p->x)?pol->points.pnts[i].y:p->y;
       p->x=(pol->points.pnts[i].z > p->x)?pol->points.pnts[i].z:p->z;
    }
 } else {
    i=0;
    copyPoint3D(p,&(pol->points.pnts[i]));
    for(i=1; i<pol->num_points; i++) {
       p->x=(pol->points.refs[i]->x > p->x)?pol->points.refs[i]->x:p->x;
       p->y=(pol->points.refs[i]->y > p->x)?pol->points.refs[i]->y:p->y;
       p->x=(pol->points.refs[i]->z > p->x)?pol->points.refs[i]->z:p->z;
    }
 }
 return p;
}

 
Point3D * minPolygon(Polygon * pol)
{
 Point3D * p;
 int i;
 p = newPoint3D(0, 0 , 0);
 if(pol==NULL || pol->num_points<=0) 
   return NULL
 ;
 if(pol->points_type==POL_POINT_VALS) {
    i=0;
    copyPoint3D(p,&(pol->points.pnts[i]));
    for(i=1; i<pol->num_points; i++) {
       p->x=(pol->points.pnts[i].x < p->x)?pol->points.pnts[i].x:p->x;
       p->y=(pol->points.pnts[i].y < p->x)?pol->points.pnts[i].y:p->y;
       p->x=(pol->points.pnts[i].z < p->x)?pol->points.pnts[i].z:p->z;
    }
 } else {
    i=0;
    copyPoint3D(p,&(pol->points.pnts[i]));
    for(i=1; i<pol->num_points; i++) {
       p->x=(pol->points.refs[i]->x < p->x)?pol->points.refs[i]->x:p->x;
       p->y=(pol->points.refs[i]->y < p->x)?pol->points.refs[i]->y:p->y;
       p->x=(pol->points.refs[i]->z < p->x)?pol->points.refs[i]->z:p->z;
    }
 }
 return p;
}

/*----------------------------------------------------------------------*/

