/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:  polymesh.c    Version: 0.5                  Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: 
  Tipos/Clases:       
  Funciones:   
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "extrafuncs.h"
#include "common3d.h"
#include "libpolymesh.h"

/*------------------  Object PolyMesh ----------------------------------*/

PolyMeshObj * newPolyMesh(char *name, SRGB * color)
{
 PolyMeshObj* pmesh=(PolyMeshObj *) malloc(sizeof(PolyMeshObj));
 pmesh->name=(char *) malloc(sizeof(char)*2);
 pmesh->name[0]='\0';
 changePolyMeshObjName(pmesh,name);
 pmesh->color=color; 
 pmesh->num_points=0;
 pmesh->points=NULL;
 pmesh->num_polygons=0;
 pmesh->polygons=NULL;
 pmesh->surface=NULL;
 return pmesh; 
}

PolyMeshObj * deletePolyMesh(PolyMeshObj * pmesh)
{
 if(pmesh==NULL) 
    return NULL
 ;
 if(pmesh->num_polygons>0) {
   for(;pmesh->num_polygons>0;pmesh->num_polygons--) 
      deletePolygon(pmesh->polygons[pmesh->num_polygons-1])
   ;
   free(pmesh->polygons);
 }
 if(pmesh->num_points>0) {
   for(;pmesh->num_points>0;pmesh->num_points--) 
      deletePoint3D(pmesh->points[pmesh->num_points-1])
   ;
   free(pmesh->points);
 }
 deletePoint3D(pmesh->color);
 deleteSurface(pmesh->surface);
 pmesh=NULL;
 return pmesh;
}

PolyMeshObj * changePolyMeshObjName(PolyMeshObj * pmesh, char * name)
{
 if(pmesh==NULL) 
    return NULL
 ;
 if(name!=NULL) {
    pmesh->name=(char *) realloc(pmesh->name,strlen(name)*sizeof(char));
    strcpy(pmesh->name,name);
    _xstrlwr(pmesh->name);
 }  
 return pmesh; 
}

int addPolyMeshObjPolygon(PolyMeshObj * pmesh, Polygon * pol)
{
 Polygon **polygons;
 if(pmesh==NULL || pol==NULL)  
    return -1 
 ;
 pmesh->num_polygons++;
 polygons=(Polygon **) realloc (pmesh->polygons,sizeof(Polygon*)*(pmesh->num_polygons));
 if(polygons!=NULL) {
/*printf("adding polygon\n");*/
   polygons[pmesh->num_polygons-1]=pol;
   pmesh->polygons=polygons;
 } else  {
/*printf("not adding polygon\n");*/
   pmesh->num_polygons--;
 }
 return pmesh->num_polygons;
}

int delPolyMeshObjPolygon(PolyMeshObj * pmesh)
{
 Polygon *pol;
 if(pmesh==NULL)  
    return -1 
 ;
 if(pmesh->num_polygons==0)
   return 0
 ;
 pmesh->num_polygons--;
 pol=pmesh->polygons[pmesh->num_polygons];
 deletePolygon(pol);
 pmesh->polygons=(Polygon **) realloc (pmesh->polygons,sizeof(Polygon*)*(pmesh->num_polygons));
 return pmesh->num_polygons;
}

int insertPolyMeshObjPolygon(PolyMeshObj * pmesh, Polygon * pol,int pos)
{
 Polygon **polygons;
 Polygon *np,*tmpp;
 if(pmesh==NULL || pol==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=pmesh->num_polygons)
   return addPolyMeshObjPolygon(pmesh,pol)
 ;
 pmesh->num_polygons++;
 polygons=(Polygon **) realloc (pmesh->polygons,sizeof(Polygon*)*(pmesh->num_polygons));
 if(polygons!=NULL) {
   for(np=pol;pos<pmesh->num_polygons; pos++) {
      tmpp=polygons[pos];
      polygons[pos]=np;
      np=tmpp;
   }
   pmesh->polygons=polygons;
 } else 
   pmesh->num_polygons--
 ;
 return pmesh->num_polygons;
}

int removePolyMeshObjPolygon(PolyMeshObj * pmesh, int pos)
{
 Polygon **polygons;
 Polygon *np,*tmpp,*pol;
 int i;
 if(pmesh==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=pmesh->num_polygons)
   return delPolyMeshObjPolygon(pmesh)
 ;
 polygons=pmesh->polygons;
 pol=polygons[pos];
 for(i=pmesh->num_polygons-2,np=polygons[i+1];i>=pos;i--) {
    tmpp=polygons[i];
    polygons[i]=np;
    np=tmpp;
 }
 deletePolygon(pol);
 pmesh->num_polygons--;
 pmesh->polygons=(Polygon **) realloc (polygons,sizeof(Polygon*)*(pmesh->num_polygons));
 return pmesh->num_polygons;
}

int addPolyMeshObjPoint(PolyMeshObj * pmesh, Point3D * p)
{
 Point3D **pnts;
 if(pmesh==NULL || p==NULL)  
    return -1 
 ;
 pmesh->num_points++;
 pnts=(Point3D **) realloc (pmesh->points,sizeof(Point3D*)*(pmesh->num_points));
 if(pnts!=NULL) {
/*printf(" adding polygon\n");*/
   pnts[pmesh->num_points-1]=p;
   pmesh->points=pnts;
 } else { 
/*printf("not adding polygon\n");*/

   pmesh->num_points--;
 }
 return pmesh->num_points;
}

int delPolyMeshObjPoint(PolyMeshObj * pmesh)
{
 Point3D *p;
 Polygon *pol;
 int i,j;
 if(pmesh==NULL)  
    return -1 
 ;
 if(pmesh->num_points==0)
   return 0
 ;
 pmesh->num_points--;
 p=pmesh->points[pmesh->num_points];
 
 /* eliminar punto de los polygonos !!! */
 for(i=0;i<pmesh->num_polygons; i++) { 
    pol=pmesh->polygons[i];    
    if(pol->pol_type==POL_POINT_REFS) {
      for(j=0; j<pol->num_points; j++) {
         if(pol->points.refs[j]==p) 
           removePolygonPoint(pol,j)
         ;
      }
    } 
 }
 /* eliminar punto ahora si */
 deletePoint3D(p);
 pmesh->points=(Point3D **) realloc (pmesh->points,sizeof(Point3D*)*(pmesh->num_points));
 return pmesh->num_points;
}

int insertPolyMeshObjPoint(PolyMeshObj * pmesh, Point3D * p,int pos)
{
 Point3D **points;
 Point3D *np,*tmpp;
 if(pmesh==NULL || p==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=pmesh->num_points)  
   return addPolyMeshObjPoint(pmesh,p)
 ;
 pmesh->num_points++;
 points=(Point3D **) realloc (pmesh->points,sizeof(Point3D*)*(pmesh->num_points));
 if(points!=NULL) {
   for(np=p;pos<pmesh->num_points; pos++) {
      tmpp=points[pos];
      points[pos]=np;
      np=tmpp;
   }
   pmesh->points=points;
 } else 
   pmesh->num_points--
 ;
 return pmesh->num_points;
}

int removePolyMeshObjPoint(PolyMeshObj * pmesh, int pos)
{
 Point3D **points;
 Point3D *np,*tmpp,*p;
 Polygon *pol;
 int i,j;
 if(pmesh==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=pmesh->num_points)
   return delPolyMeshObjPoint(pmesh)
 ;
 points=pmesh->points;
 p=points[pos];
 /* eliminar punto de los polygonos !!! */
 for(i=0;i<pmesh->num_polygons; i++) { 
    pol=pmesh->polygons[i];    
    if(pol->pol_type==POL_POINT_REFS) {
      for(j=0; j<pol->num_points; j++) {
         if(pol->points.refs[j]==p) 
           removePolygonPoint(pol,j)
         ;
      }
    } 
 }

 /* ahora si se borra el punto */
 for(i=pmesh->num_points-2,np=points[i+1];i>=pos;i--) {
    tmpp=points[i];
    points[i]=np;
    np=tmpp;
 }
 deletePoint3D(p);
 pmesh->num_points--;
 pmesh->points=(Point3D **) realloc (points,sizeof(Point3D*)*(pmesh->num_points));
 return pmesh->num_polygons;
}

int addPolyMeshObjPointEx(PolyMeshObj * pmesh, Point3D * p)
{
 int i;
 if(pmesh==NULL || p==NULL )  
    return -1 
 ;
 for(i=0;i<pmesh->num_points && !eqPoint3D(pmesh->points[i],p); i++) 
 ;
 if(i>=pmesh->num_points) {
   return addPolyMeshObjPoint(pmesh,p);
 }
 return i;  
}

int addPolyMeshObjPolygonPoint(PolyMeshObj * pmesh, Point3D * p)
{ 
 Polygon *pol;
 if(pmesh==NULL || p==NULL)
   return -1 
 ;
 if(pmesh->num_polygons==0) 
   return -1
 ;
 pol=pmesh->polygons[pmesh->num_polygons-1];
 return  addPolygonPoint(pol,p);
}

int delPolyMeshObjPolygonPoint(PolyMeshObj * pmesh)
{
 Polygon *pol;
 if(pmesh==NULL)
   return -1 
 ;
 if(pmesh->num_polygons==0) 
   return -1
 ;
 pol=pmesh->polygons[pmesh->num_polygons-1];
 return  delPolygonPoint(pol);
}

int insertPolyMeshObjPolygonPoint(PolyMeshObj * pmesh, int pol_num, Point3D * p, int point_num) 
{
 Polygon *pol;
 if(pmesh==NULL || pol_num<0 || p==NULL || point_num<0)
   return -1 
 ;
 if(pmesh->num_polygons==0) 
   return -1
 ;
 if(pol_num>=pmesh->num_polygons-1) {
   pol=pmesh->polygons[pmesh->num_polygons-1];
   return insertPolygonPoint(pol,p,point_num);
 }
 pol=pmesh->polygons[pol_num];
 return  insertPolygonPoint(pol,p,point_num);
}

int removePolyMeshObjPolygonPoint(PolyMeshObj * pmesh, int pol_num, int point_num)
{
 Polygon *pol;
 if(pmesh==NULL || pol_num<0 || point_num<0)
   return -1 
 ;
 if(pmesh->num_polygons==0) 
   return -1
 ;
 if(pol_num>=pmesh->num_polygons-1) {
    pol=pmesh->polygons[pmesh->num_polygons-1];
    return  removePolygonPoint(pol,point_num);
 }  
 pol=pmesh->polygons[pol_num];
 return  removePolygonPoint(pol,point_num);
}

Point3D * maxPolyMeshObj(PolyMeshObj * pmesh)
{
 Point3D * p;
 int i;
 p = newPoint3D(0,0,0);
 if(pmesh==NULL || pmesh->num_points<=0) 
   return NULL
 ;
 i=0;
 copyPoint3D(p,pmesh->points[i]);
 for(i=1; i<pmesh->num_points; i++) {
    p->x=(pmesh->points[i]->x > p->x)?pmesh->points[i]->x:p->x;
    p->y=(pmesh->points[i]->y > p->y)?pmesh->points[i]->y:p->y;
    p->z=(pmesh->points[i]->z > p->z)?pmesh->points[i]->z:p->z;
 }
 return p;
}

Point3D * minPolyMeshObj(PolyMeshObj * pmesh)
{
 Point3D * p ;
 int i;
 p = newPoint3D(0,0,0);
 if(pmesh==NULL || pmesh->num_points<=0)
   return NULL
 ;
 i=0;
 copyPoint3D(p,pmesh->points[i]);
 for(i=1; i<pmesh->num_points; i++) {
    p->x=(pmesh->points[i]->x < p->x)?pmesh->points[i]->x:p->x;
    p->y=(pmesh->points[i]->y < p->y)?pmesh->points[i]->y:p->y;
    p->z=(pmesh->points[i]->z < p->z)?pmesh->points[i]->z:p->z;
 }
 return p;
}

/*----------------------------------------------------------------------*/

