/*------------------------------------------------------------------------
                                /\
                             __/__\__
                            /\      /\
                           /  \    /  \ 
                          /    \  /    \
                         /______\/______\
                          /____________\
                          M   M OOOO X  X
                          M M M 0  0  XX
                          M   M OOOO X  X
 
          MOX - Centro de Computacion Avanzada en Ingenieria.
                        Facultad de Ingenieria
                       Universidad de los Andes
                         Bogota, Colombia.
------------------------------------------------------------------------*/

/*************************************************************************
  Nombre:  scene.c         Version: 0.5                  Fecha: Abril-1996
 -------------------------------------------------------------------------
  Descripcion: 
  Tipos/Clases:       
  Funciones:   
  Observaciones:  

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Autor: Daniel Barrero                                  Fecha: Abril-1996 
**************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "extrafuncs.h"
#include "common3d.h"
#include "libpolymesh.h"

/*------------------  Object SceneObj ----------------------------------*/

SceneObj * newScene(char *name, SRGB * color)
{
 SceneObj* scene=(SceneObj *) malloc(sizeof(SceneObj));
 scene->name=(char *) malloc(sizeof(char)*2);
 scene->name[0]='\0';
 changeSceneObjName(scene,name);
 scene->color=color; 
 scene->num_objects=0;
 scene->objects=NULL;

 return scene; 
}

SceneObj * deleteScene(SceneObj * scene)
{
 if(scene==NULL) 
    return NULL
 ;
 if(scene->num_objects>0) {
   for(;scene->num_objects>0;scene->num_objects--) 
      deletePolyMesh(scene->objects[scene->num_objects-1])
   ;
   free(scene->objects);
 }
 free(scene->color);
 scene=NULL;
 return scene;
}

SceneObj * changeSceneObjName(SceneObj * scene, char * name)
{
 if(scene==NULL) 
    return NULL
 ;
 if(name!=NULL) {
    scene->name=(char *) realloc(scene->name,strlen(name)*sizeof(char));
    strcpy(scene->name,name);
    _xstrlwr(scene->name);
 }  
 return scene; 
}

int addSceneObjPolyMesh(SceneObj * scene, PolyMeshObj * pmesh)
{
 PolyMeshObj **pmeshes;
 if(scene==NULL || pmesh==NULL)  
    return -1 
 ;
 scene->num_objects++;
 pmeshes=(PolyMeshObj **) realloc (scene->objects,sizeof(PolyMeshObj*)*(scene->num_objects));
 if(pmeshes!=NULL) {
   pmeshes[scene->num_objects-1]=pmesh;
   scene->objects=pmeshes;
 } else 
   scene->num_objects--
 ;
 return scene->num_objects;
}

int delSceneObjPolyMesh(SceneObj * scene)
{
 PolyMeshObj *pmesh;
 if(scene==NULL)  
    return -1 
 ;
 if(scene->num_objects==0)
   return 0
 ;
 scene->num_objects--;
 pmesh=scene->objects[scene->num_objects];
 deletePolyMesh(pmesh);
 scene->objects=(PolyMeshObj **) realloc (scene->objects,sizeof(PolyMeshObj*)*(scene->num_objects));
 return scene->num_objects;
}

int insertSceneObjPolyMesh(SceneObj * scene, PolyMeshObj * pmesh, int pos)
{
 PolyMeshObj **pmeshes;
 PolyMeshObj *np,*tmpp;
 if(scene==NULL || pmesh==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=scene->num_objects)
   return addSceneObjPolyMesh(scene,pmesh)
 ;
 scene->num_objects++;
 pmeshes=(PolyMeshObj **) realloc (scene->objects,sizeof(PolyMeshObj*)*(scene->num_objects));
 if(pmeshes!=NULL) {
   for(np=pmesh;pos<scene->num_objects; pos++) {
      tmpp=pmeshes[pos];
      pmeshes[pos]=np;
      np=tmpp;
   }
   scene->objects=pmeshes;
 } else 
   scene->num_objects--
 ;
 return scene->num_objects;
}

int removeSceneObjPolyMesh(SceneObj * scene, int pos)
{
 PolyMeshObj **pmeshes;
 PolyMeshObj *np,*tmpp,*pol;
 int i;
 if(scene==NULL || pos <0 )  
    return -1 
 ;
 if(pos>=scene->num_objects)
   return delSceneObjPolyMesh(scene)
 ;
 pmeshes=scene->objects;
 pol=pmeshes[pos];  
 for(i=scene->num_objects-2,np=pmeshes[i+1];i>=pos;i--) {
    tmpp=pmeshes[i];
    pmeshes[i]=np;
    np=tmpp;
 }
 deletePolyMesh(pol);
 scene->num_objects--;
 scene->objects=(PolyMeshObj **) realloc (pmeshes,sizeof(PolyMeshObj*)*(scene->num_objects));
 return scene->num_objects; 
}

Point3D * maxSceneObj(SceneObj * scene)
{
 Point3D * p,* p1;
 int i;
 p = newPoint3D(0,0,0);
 p1=newPoint3D(0,0,0);
 if(scene==NULL || scene->num_objects<=0)
   return NULL
 ;
 i=0;
 copyPoint3D(p,maxPolyMeshObj(scene->objects[i]));
 for(i=1; i<scene->num_objects; i++) {
    copyPoint3D(p1,maxPolyMeshObj(scene->objects[i]));
    p->x=(p1->x > p->x)?p1->x:p->x;
    p->y=(p1->y > p->y)?p1->y:p->y;
    p->z=(p1->z > p->z)?p1->z:p->z;
 }
 return p;
}

Point3D * minSceneObj(SceneObj * scene)
{
 Point3D * p, * p1;
 int i;
 p = newPoint3D (0,0,0);
 p1=newPoint3D (0,0,0);
 if(scene==NULL || scene->num_objects<=0)
   return NULL
 ;
 i=0;
 copyPoint3D(p,minPolyMeshObj(scene->objects[i]));
 for(i=1; i<scene->num_objects; i++) {
    copyPoint3D(p1,minPolyMeshObj(scene->objects[i]));
    p->x=(p1->x < p->x)?p1->x:p->x;
    p->y=(p1->y < p->y)?p1->y:p->y;
    p->z=(p1->z < p->z)?p1->z:p->z;
 }
 return p;
}
/*----------------------------------------------------------------------*/


